__description__ = "Small wrapper for HexWay Hive API "
__author__ = "@Cur1iosity"
__email__ = "Cur1iosity@protonmail.com"
__licence__ = "MIT"
__version__ = "0.1.0"

from hive_client import Hive

__all__ = [
        'Hive',
]
